#include <assert.h>
#include "FaspManagerSample.h"

/**
 * Transfer parameters
 */
#define TARGET_RATE          10000    /* Target (max) transfer rate in Kbps */
#define MIN_RATE             5000     /* Minimum transfer rate in Kbps - adaptive rate only */
#define CREATE_DIRS          false    /* Create the target directory if it does not exist   */

int main()
{
    Aspera::FaspManager::JobOrder *job_defn;
    Aspera::FaspManager::Location *src;
    Aspera::FaspManager::Location *dst;
    Aspera::FaspManager::XferParams *xfer_params;
    Aspera::FaspManager::Manager *manager;

    printf("\nPersistent Download Sample:\n\n");

    /*
    * Configure the fasp Manager environment. The fasp manager environment
    * can be configured using the set methods provided by Environment class.
    * Depending on your deployment scenario you need to set one or more of
    * these. Refer to the documentation bundled with the SDK for more details.
    */
    // AscpPath is no longer required, Environment will now attempt to find ascp
    //Aspera::FaspManager::Environment::SetFasp2AscpPath("C:\\Program Files\\Aspera\\Enterprise Server\\bin\\ascp.exe");
    //Aspera::FaspManager::Environment::SetPortFilePath("C:\\Program Files\\Aspera\\Enterprise Server\\var\\run\\aspera\\fmsample.optport");

    /*
     * Setup the Local and Remote endpoints for a download
     */
    dst = Aspera::FaspManager::Location::CreateLocal( );
    src = Aspera::FaspManager::Location::CreateRemote( "demo.asperasoft.com", "aspera" , "demoaspera" );

    /* Setup the transfer parameters */
    xfer_params = new Aspera::FaspManager::XferParams( );
    xfer_params->encryption = Aspera::FaspManager::ENCRYPT_AES128;
    xfer_params->target_rate = TARGET_RATE; /* Transfers never exceed this rate  */
    xfer_params->min_rate = MIN_RATE;
    xfer_params->policy = Aspera::FaspManager::POLICY_FAIR;
    xfer_params->create_dirs = CREATE_DIRS;
    xfer_params->cookie = "my sample cookie";
    xfer_params->resume_check = Aspera::FaspManager::RESUME_NONE;
    xfer_params->persist = true;

    /* Setup the job order */
    job_defn = new Aspera::FaspManager::JobOrder( );
    job_defn->src_location = src;
    job_defn->dst_location = dst;
    job_defn->xfer_params = xfer_params;

    const char* job_id;

    /* Setup a callback function */
    Aspera::FaspManager::CallbackFunctor<MyCallbackClass> *callback;
    MyCallbackClass *callbackobj = new MyCallbackClass();
    callback = new Aspera::FaspManager::CallbackFunctor<MyCallbackClass>( callbackobj, &MyCallbackClass::ReceiveCallback );

    manager = Aspera::FaspManager::Manager::GetInstance();
    manager->SetDebugLevel(1);
    // manager->AddGlobalCallback(callback); // This callback fires for all events for all jobs/sessions

    /* If using fasp manager on the server, uncomment the following. */
    //manager->SetManagementPortPublic(true);

    /* Launch the job */
    job_id = manager->LaunchJob ( job_defn, callback ); // A callback specified here will only fire for this job

    /*  Wait for session to start  */
    while (!callbackobj->isSessionReady()) {
	callbackobj->sleep(1000);
    }

    /*  Queue files to be downloaded */
    manager->AddSourcePath(job_id, "aspera-test-dir-small/10MB.1", "downloaded_small_test_file");
    manager->AddSourcePath(job_id, "aspera-test-dir-small/10MB.2", "downloaded_small_test_file2");

    /* Let it run until one file transferred or an error occurs  (the session doesn't stop until we tell it to) */
    while (callbackobj->getFileTransferCount() != 2) {
	callbackobj->sleep(1000);
    }

    /* Stop session */
    manager->StopPersistentJob(job_id);

    /* Wait for session to stop  */
    while (callbackobj->isSessionReady()) {
	callbackobj->sleep(1000);
    }

    printf("\nPress Enter to end the demo...\n");
    getc( stdin );

    /* Cleanup and close mgr */
    manager->Destroy();

    return 0;
}

